// Defining the hierarchical data in JSON format
var treeData = [
  {
    "name": "Dog Breed",
    "parent": "null",
    "children": [
      {
        "name": "Retriever",
        "parent": "Dog Breed",
        "type": "grey",
             "level": "blue",
        "children": [
          {
            "name": "Golden Retriever",
            "parent": "Retriever"
          },
          {
            "name": "White Retriever",
            "parent": "Retriever"
          }
        ]
      },
      {
        "name": "Bull Dog",
        "parent": "Dog Breed",
        "color": "yellow",
        "children": [
          {
            "name": "French BullDog",
            "parent": "Bull Dog"
          },
          {
            "name": "American BullDog",
            "parent": "Bull Dog"
          }
        ]
      },
	   {
        "name": "Poodle",
        "parent": "Dog Breed",
       "children": [
          {
            "name": "Standard Poodle",
            "parent": "Poodle",
             "children": [
          {
            "name": "Small Poodle",
            "parent": "Standard Poodle",
            "type": "red",
             "level": "black",
            
          }]
          },
          {
            "name": "Toy Poodle",
            "parent": "Poodle"
          },
          {
            "name": "Moyen Poodle",
            "parent": "Poodle"
          }
        ]
      }
    ]
  }
];


// *Generate the tree diagram*
var margin = {top: 0, right: 120, bottom: 0, left: 200},//dimensions for root node of the tree
	width = 960 - margin.right - margin.left,
	height = 400 - margin.top - margin.bottom;
	
var i = 0,
	duration = 750,
	root;
//Defining the tree layout and the diagonal projection for links
var tree = d3.layout.tree()
	.size([height, width]);

var diagonal = d3.svg.diagonal()
	.projection(function(d) { return [d.y, d.x]; });
//Create the SVG and translate it based on the margins
var svg = d3.select("body").append("svg")
	.attr("width", width + margin.right + margin.left)
	.attr("height", height + margin.top + margin.bottom)
  .append("g")
	.attr("transform", "translate(" + margin.left + "," + margin.top + ")");
//Set the initial position of the root node and update the tree
root = treeData[0];
root.x0 = height / 2;
root.y0 = 0;
  
update(root);
//Set the height of the SVG based on the height of the tree
d3.select(self.frameElement).style("height", "500px");

function update(source) {

  //Compute the new tree layout
  var nodes = tree.nodes(root).reverse(),
	  links = tree.links(nodes);

  //Normalize for fixed-depth
  nodes.forEach(function(d) { d.y = d.depth * 180; });

  //Update the nodes
  var node = svg.selectAll("g.node")
	  .data(nodes, function(d) { return d.id || (d.id = ++i); });

  //Enter any new nodes at the parent's previous position
  var nodeEnter = node.enter().append("g")
	  .attr("class", "node")
	  .attr("transform", function(d) { return "translate(" + source.y0 + "," + source.x0 + ")"; })
	  .on("click", click);

  nodeEnter.append("circle")
	  .attr("r", 1e-6)
	  .style("fill", function(d) { return d._children ? "lightsteelblue" : "#fff"; });

  nodeEnter.append("text")
	  .attr("x", function(d) { return d.children || d._children ? -13 : 13; })
	  .attr("dy", ".35em")
	  .attr("text-anchor", function(d) { return d.children || d._children ? "end" : "start"; })
	  .text(function(d) { return d.name; })
	  .style("fill-opacity", 1e-6);

  //Transition nodes to their new position
  var nodeUpdate = node.transition()
	  .duration(duration)
	  .attr("transform", function(d) { return "translate(" + d.y + "," + d.x + ")"; });

  nodeUpdate.select("circle")
	  .attr("r", 15)
	  .style("fill", function(d) { return d._children ? "yellow" : "teal"; });

  nodeUpdate.select("text")
	  .style("fill-opacity", 1);

  //Transition to the exiting nodes to parent's new position
  var nodeExit = node.exit().transition()
	  .duration(duration)
	  .attr("transform", function(d) { return "translate(" + source.y + "," + source.x + ")"; })
	  .remove();

  nodeExit.select("circle")
	  .attr("r", 1e-6);

  nodeExit.select("text")
	  .style("fill-opacity", 1e-6);

  //Update the link
  var link = svg.selectAll("path.link")
	  .data(links, function(d) { return d.target.id; });

  //Enter any new links at the parent's previous position
  link.enter().insert("path", "g")
	  .attr("class", "link")
	  .attr("d", function(d) {
		var o = {x: source.x0, y: source.y0};
		return diagonal({source: o, target: o});
	  });

  //Transition links to their new position
  link.transition()
	  .duration(duration)
	  .attr("d", diagonal);

  //Transition exiting nodes to the parent's new position
  link.exit().transition()
	  .duration(duration)
	  .attr("d", function(d) {
		var o = {x: source.x, y: source.y};
		return diagonal({source: o, target: o});
	  })
	  .remove();

  // Transistion old positions
  nodes.forEach(function(d) {
	d.x0 = d.x;
	d.y0 = d.y;
  });
}

// Enter the new nodes at the parent's and children
function click(d) {
  if (d.children) {
	d._children = d.children;
	d.children = null;
  } else {
	d.children = d._children;
	d._children = null;
  }
  update(d);
}
